<?php
/*	Project:	EQdkp-Plus
 *	Package:	EQdkp-plus
 *	Link:		http://eqdkp-plus.eu
 *
 *	Copyright (C) 2006-2016 EQdkp-Plus Developer Team
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Affero General Public License as published
 *	by the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Affero General Public License for more details.
 *
 *	You should have received a copy of the GNU Affero General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

if ( !defined('EQDKP_INC') ){
	header('HTTP/1.0 404 Not Found');exit;
}

class xenforo_bridge extends bridge_generic {

	public static $name = 'XenForo';

	private $passwordIterations = 10;

	public $data = array(
		//Data
		'groups' => array( //Where I find the Usergroup
			'table'	=> 'user_group', //without prefix
			'id'	=> 'user_group_id',
			'name'	=> 'title',
		),
		'user_group' => array( //Zuordnung User zu Gruppe
			'table'	=> 'user_group_relation',
			'group'	=> 'user_group_id',
			'user'	=> 'user_id',
			'QUERY'	=> '',
		),
		'user'	=> array( //User
			'table'	=> 'user',
			'id'	=> 'user_id',
			'name'	=> 'username',
			'where'	=> 'username',
			'password' => 'user_id',
			'email'	=> 'email',
			'salt'	=> '',
			'QUERY'	=> '',
			'FUNCTION' => 'get_user',
		),
	);

	public $settings = array(
	);

	public function check_password($password, $hash, $strSalt = '', $strUsername = "", $arrUserdata=array()){
		$strQuery = "SELECT * FROM ".$this->prefix."user_authenticate WHERE user_id=?";
		$objQuery = $this->bridgedb->prepare($strQuery)->execute($hash);

		if ($objQuery){
			$arrAllResults = $objQuery->fetchAllAssoc();
			foreach($arrAllResults as $arrResult){
				$arrAuthData = unserialize_noclasses($arrResult['data']);
				$scheme = $arrResult['scheme_class'];

				$blnResult = $this->_handle_logins($scheme, $password, $hash, $arrAuthData);
				if ($blnResult) return true;
			}
		}

		return false;
	}

	private function _handle_logins($scheme, $password, $hash, $arrAuthData){
		switch($scheme){
			case "XenForo_Authentication_Core12":
				$passwordHash = new XenForo_PasswordHash($this->passwordIterations, false);
				return $passwordHash->CheckPassword($password, $arrAuthData['hash']);

			break;

			case "XenForo_Authentication_Core":
				$userHash = hash('sha256', hash('sha256', $password).$arrAuthData['salt']);
				if ($userHash === $arrAuthData['hash']) return true;

				$userHash = sha1(sha1($password).$arrAuthData['salt']);
				return ($userHash === $arrAuthData['hash']);
			break;

			case "XenForo_Authentication_IPBoard":
			case "XenForo_Authentication_MyBb":
				$userHash = md5(md5($arrAuthData['salt']) . md5($password));
				return ($userHash === $arrAuthData['hash']);
			break;

			case "XenForo_Authentication_PhpBb3":
				$passwordHash = new XenForo_PasswordHash(8, true);
				return $passwordHash->CheckPassword($password, $arrAuthData['hash']);
			break;

			case "XenForo_Authentication_vBulletin":
				$userHash = md5(md5($password) . $arrAuthData['salt']);
				return ($userHash === $arrAuthData['hash']);
			break;
		}

		return false;
	}


}

#
# Portable PHP password hashing framework.
#
# Version 0.3 / modified for XenFOro.
#
# Written by Solar Designer <solar at openwall.com> in 2004-2006 and placed in
# the public domain.  Revised in subsequent years, still public domain.
#
# There's absolutely no warranty.
#
# The homepage URL for this framework is:
#
#	http://www.openwall.com/phpass/
#
# Please be sure to update the Version line if you edit this file in any way.
# It is suggested that you leave the main version number intact, but indicate
# your project name (after the slash) and add your own revision information.
#
# Please do not change the "private" password hashing method implemented in
# here, thereby making your hashes incompatible.  However, if you must, please
# change the hash type identifier (the "$P$") to something different.
#
# Obviously, since this code is in the public domain, the above are not
# requirements (there can be none), but merely suggestions.
#
if (!class_exists("XenForo_PasswordHash")){
	class XenForo_PasswordHash {
		var $itoa64;
		var $iteration_count_log2;
		var $portable_hashes;
		var $random_state;

		public function __construct($iteration_count_log2, $portable_hashes)
		{
			$this->itoa64 = './0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';

			if ($iteration_count_log2 < 4 || $iteration_count_log2 > 31)
				$iteration_count_log2 = 8;
			$this->iteration_count_log2 = $iteration_count_log2;

			$this->portable_hashes = $portable_hashes;

			$this->random_state = microtime();
			if (function_exists('getmypid'))
				$this->random_state .= getmypid();
		}

		function get_random_bytes($count)
		{
			$output = '';

			if (function_exists('openssl_random_pseudo_bytes')
				&& (substr(PHP_OS, 0, 3) != 'WIN' || version_compare(phpversion(), '5.3.4', '>='))
			)
			{
				$output = openssl_random_pseudo_bytes($count);
			}
			else if (function_exists('mcrypt_create_iv') && version_compare(phpversion(), '5.3.0', '>='))
			{
				$output = mcrypt_create_iv($count, MCRYPT_DEV_URANDOM);
			}
			else if (@is_readable('/dev/urandom') &&
				($fh = @fopen('/dev/urandom', 'rb'))) {
				$output = fread($fh, $count);
				fclose($fh);
			}

			if (strlen($output) < $count) {
				$output = '';
				for ($i = 0; $i < $count; $i += 16) {
					$this->random_state =
						md5(microtime() . $this->random_state);
					$output .=
						pack('H*', md5($this->random_state));
				}
				$output = substr($output, 0, $count);
			}

			return $output;
		}

		function encode64($input, $count)
		{
			$output = '';
			$i = 0;
			do {
				$value = ord($input[$i++]);
				$output .= $this->itoa64[$value & 0x3f];
				if ($i < $count)
					$value |= ord($input[$i]) << 8;
				$output .= $this->itoa64[($value >> 6) & 0x3f];
				if ($i++ >= $count)
					break;
				if ($i < $count)
					$value |= ord($input[$i]) << 16;
				$output .= $this->itoa64[($value >> 12) & 0x3f];
				if ($i++ >= $count)
					break;
				$output .= $this->itoa64[($value >> 18) & 0x3f];
			} while ($i < $count);

			return $output;
		}

		protected function gensalt_private($input)
		{
			$output = '$P$';
			$output .= $this->itoa64[min($this->iteration_count_log2 +
				((PHP_VERSION >= '5') ? 5 : 3), 30)];
			$output .= $this->encode64($input, 6);

			return $output;
		}

		protected function crypt_private($password, $setting)
		{
			$output = '*0';
			if (substr($setting, 0, 2) == $output)
				$output = '*1';

			$id = substr($setting, 0, 3);
			# We use "$P$", phpBB3 uses "$H$" for the same thing
			if ($id != '$P$' && $id != '$H$')
				return $output;

			$count_log2 = strpos($this->itoa64, $setting[3]);
			if ($count_log2 < 7 || $count_log2 > 30)
				return $output;

			$count = 1 << $count_log2;

			$salt = substr($setting, 4, 8);
			if (strlen($salt) != 8)
				return $output;

			# We're kind of forced to use MD5 here since it's the only
			# cryptographic primitive available in all versions of PHP
			# currently in use.  To implement our own low-level crypto
			# in PHP would result in much worse performance and
			# consequently in lower iteration counts and hashes that are
			# quicker to crack (by non-PHP code).
			if (PHP_VERSION >= '5') {
				$hash = md5($salt . $password, TRUE);
				do {
					$hash = md5($hash . $password, TRUE);
				} while (--$count);
			} else {
				$hash = pack('H*', md5($salt . $password));
				do {
					$hash = pack('H*', md5($hash . $password));
				} while (--$count);
			}

			$output = substr($setting, 0, 12);
			$output .= $this->encode64($hash, 16);

			return $output;
		}

		function gensalt_extended($input)
		{
			$count_log2 = min($this->iteration_count_log2 + 8, 24);
			# This should be odd to not reveal weak DES keys, and the
			# maximum valid value is (2**24 - 1) which is odd anyway.
			$count = (1 << $count_log2) - 1;

			$output = '_';
			$output .= $this->itoa64[$count & 0x3f];
			$output .= $this->itoa64[($count >> 6) & 0x3f];
			$output .= $this->itoa64[($count >> 12) & 0x3f];
			$output .= $this->itoa64[($count >> 18) & 0x3f];

			$output .= $this->encode64($input, 3);

			return $output;
		}

		function gensalt_blowfish($input)
		{
			# This one needs to use a different order of characters and a
			# different encoding scheme from the one in encode64() above.
			# We care because the last character in our encoded string will
			# only represent 2 bits.  While two known implementations of
			# bcrypt will happily accept and correct a salt string which
			# has the 4 unused bits set to non-zero, we do not want to take
			# chances and we also do not want to waste an additional byte
			# of entropy.
			$itoa64 = './ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';

			$output = '$2a$';
			$output .= chr(ord('0') + $this->iteration_count_log2 / 10);
			$output .= chr(ord('0') + $this->iteration_count_log2 % 10);
			$output .= '$';

			$i = 0;
			do {
				$c1 = ord($input[$i++]);
				$output .= $itoa64[$c1 >> 2];
				$c1 = ($c1 & 0x03) << 4;
				if ($i >= 16) {
					$output .= $itoa64[$c1];
					break;
				}

				$c2 = ord($input[$i++]);
				$c1 |= $c2 >> 4;
				$output .= $itoa64[$c1];
				$c1 = ($c2 & 0x0f) << 2;

				$c2 = ord($input[$i++]);
				$c1 |= $c2 >> 6;
				$output .= $itoa64[$c1];
				$output .= $itoa64[$c2 & 0x3f];
			} while (1);

			return $output;
		}

		function HashPassword($password)
		{
			$random = '';

			if (CRYPT_BLOWFISH == 1 && !$this->portable_hashes) {
				$random = $this->get_random_bytes(16);
				$hash =
					crypt($password, $this->gensalt_blowfish($random));
				if (strlen($hash) == 60)
					return $hash;
			}

			if (CRYPT_EXT_DES == 1 && !$this->portable_hashes) {
				if (strlen($random) < 3)
					$random = $this->get_random_bytes(3);
				$hash =
					crypt($password, $this->gensalt_extended($random));
				if (strlen($hash) == 20)
					return $hash;
			}

			if (strlen($random) < 6)
				$random = $this->get_random_bytes(6);
			$hash =
				$this->crypt_private($password,
				$this->gensalt_private($random));
			if (strlen($hash) == 34)
				return $hash;

			# Returning '*' on error is safe here, but would _not_ be safe
			# in a crypt(3)-like function used _both_ for generating new
			# hashes and for validating passwords against existing hashes.
			return '*';
		}

		function CheckPassword($password, $stored_hash)
		{
			$hash = $this->crypt_private($password, $stored_hash);
			if ($hash[0] == '*')
				$hash = crypt($password, $stored_hash);

			return $hash == $stored_hash;
		}

		public function reverseItoA64($char)
		{
			return strpos($this->itoa64, $char);
		}
	}
}
