/**
 * plugin.js
 *
 * Copyright, Moxiecode Systems AB
 * Released under LGPL License.
 *
 * License: http://www.tinymce.com/license
 * Contributing: http://www.tinymce.com/contributing
 */

/*global tinymce:true */

tinymce.PluginManager.add('eqdkp_gallery', function(editor) {
	
	function showDialog(folderList) {
		var win, data, dom = editor.dom, imgElm = editor.selection.getNode();
		var width, height, imageListCtrl;
		
		data = {
			sort: 0,
			folder: 'system',
		};
		
		function buildSortationList(relValue) {
			var relListItems = [];

			relListItems.push({
				text: "Name ascending",
				value: 0,
				selected: relValue === 0
			});
			
			relListItems.push({
				text: "Name descending",
				value: 1,
				selected: relValue === 1
			});
			
			relListItems.push({
				text: "Date ascending",
				value: 2,
				selected: relValue === 2
			});
			
			relListItems.push({
				text: "Date descending",
				value: 3,
				selected: relValue === 3
			});

			return relListItems;
		}

		if (imgElm.nodeName == "P" && imgElm.getAttribute('class') == "system-gallery") {
			data = {
				sort: dom.getAttrib(imgElm, 'data-sort'),
				folder: dom.getAttrib(imgElm, 'data-folder'),
			};
		} else {
			imgElm = null;
		}
		
		function buildFolderList(relValue){
			var relListItems = [];
			
			tinymce.each(folderList, function(rel) {
				relListItems.push({
					text: rel.text || rel.title,
					value: rel.value,
					selected: relValue === rel.value
				});
			});
			
			return relListItems;
		}

		win = editor.windowManager.open({
			title: "Image Gallery",
			data: data,
			body: [
				{name: 'folder', type: 'listbox', label: 'Select Folder', values: buildFolderList(data.folder)},
				{name: 'sort', type: 'listbox', label: 'Sortation', values: buildSortationList(data.sort)},
			],
			onSubmit: function(e) {
				var data = e.data;
				if (data.folder != ""){
					if (imgElm) {
						dom.setAttrib(imgElm, "data-sort", data.sort);
						dom.setAttrib(imgElm, "data-folder", data.folder);
					} else {
						editor.insertContent('<p class="system-gallery" data-sort="'+data.sort+'" data-folder="'+data.folder+'"></p><p></p>');
					}
				}
				
			}
		});
	}
	
	function prepareDialog(){
		jQuery.get(editor.baseURI.source+"/plugins/eqdkp_gallery/get_folder.php"+mmocms_sid, function(retdata){
			if(retdata.status == 'ok'){
				var relListItems = [];
				
				jQuery.each(retdata.folders, function(key, value) {
					relListItems.push({
						text: value.folder.replace('*+*+*', '/'),
						value: value.folder,
						selected: 0,
					});	
				});
								
				showDialog(relListItems);
			} else {
				alert('Error while reading folder.');
			}
		}, "json");
	}

	editor.addButton('eqdkp_gallery', {
		icon: 'image',
		tooltip: 'Insert Image Gallery',
		onclick: prepareDialog,
		stateSelector: 'p.system-gallery'
	});

	editor.addMenuItem('eqdkp_gallery', {
		icon: 'image',
		text: 'Insert Image Gallery',
		onclick: prepareDialog,
		context: 'insert',
		prependToContext: true
	});
});