/**
 * plugin.js
 *
 * Copyright, Moxiecode Systems AB
 * Released under LGPL License.
 *
 * License: http://www.tinymce.com/license
 * Contributing: http://www.tinymce.com/contributing
 */

/*global tinymce:true */

tinymce.PluginManager.add('eqdkp_pagebreak_readmore', function(editor) {
	
	function insertPagebreak(){
		var win, data, dom = editor.dom, imgElm = editor.selection.getNode();
		var width, height, imageListCtrl;
		
		data = {
			title: '',
		};
		
		if (imgElm.nodeName == "HR" && imgElm.getAttribute('class') == 'system-pagebreak') {
			data = {
				title: dom.getAttrib(imgElm, 'title'),
			};
		} else {
			imgElm = null;
		}
		
		win = editor.windowManager.open({
			title: "Insert Pagebreak",
			data: data,
			body: [
				{name: 'title', type: 'textbox', label: 'Pagetitle'},
			],
			onSubmit: function(e) {
				var data = e.data;
				
				if (imgElm) {
					dom.setAttrib(imgElm, "title", data.title);
				} else {
					editor.insertContent('<hr class="system-pagebreak" title="'+data.title+'" /><p></p>');
				}
	
			}
		});
	}
	
	function insertReadmore(){
		var win, data, dom = editor.dom, imgElm = editor.selection.getNode();
		var width, height, imageListCtrl;
		
		var content = editor.getContent();
		if (content.match(/<hr\s+id=("|')system-readmore("|')\s*\/*>/i)) {
			alert('There is already a Read more... link that has been inserted. Only one such link is permitted. Use {Pagebreak} to split the page up further.');
			return false;
		} else {
			editor.insertContent('<hr id="system-readmore" /><p></p>');
		}
	}
	
	editor.addButton('eqdkp_readmore', {
		icon: 'readmore',
		tooltip: 'Insert Readmore-Divider',
		onclick: insertReadmore,
	});
	
	editor.addButton('eqdkp_pagebreak', {
		icon: 'pagebreak',
		tooltip: 'Insert Pagebreak',
		onclick: insertPagebreak,
		stateSelector: 'hr.system-pagebreak',
	});
	
	editor.addMenuItem('eqdkp_readmore', {
		icon: 'readmore',
		text: 'Insert Readmore-Divider',
		onclick: insertReadmore,
		context: 'insert',
		prependToContext: true
	});

	editor.addMenuItem('eqdkp_pagebreak', {
		icon: 'pagebreak',
		text: 'Insert Pagebreak',
		onclick: insertPagebreak,
		context: 'insert',
		prependToContext: true
	});
});