/**
 * plugin.js
 *
 * Copyright, Moxiecode Systems AB
 * Released under LGPL License.
 *
 * License: http://www.tinymce.com/license
 * Contributing: http://www.tinymce.com/contributing
 */

/*global tinymce:true */

tinymce.PluginManager.add('eqdkp_pageobject', function(editor) {
	
	function insertPageobject(){
		var win, data, dom = editor.dom, imgElm = editor.selection.getNode();
		var width, height, imageListCtrl;
		var last = '';
		
		function buildPageobjectList(relValue){
			var relListItems = [];

			if (pageobjects == undefined) return relListItems;
			
			tinymce.each(pageobjects, function(rel, key) {
				relListItems.push({
					text: rel,
					value: key,
					selected: relValue === rel
				});
				
				last = key;
			});
			
			return relListItems;
		}
		
		data = {
			title: '',
		};
		
		if (imgElm.nodeName == "P" && imgElm.getAttribute('class') == 'system-article') {
			data = {
				title: dom.getAttrib(imgElm, 'title'),
			};
		} else {
			var items = buildPageobjectList();
			imgElm = null;
			data = {
				title: last,
			};
		}
		
		win = editor.windowManager.open({
			title: "Insert Pageobject",
			data: data,
			body: [
				{name: 'title', type: 'listbox', label: 'Select Pageobject', values: buildPageobjectList(data.title)},
			],
			onSubmit: function(e) {
				var data = e.data;
				
				if (imgElm) {
					dom.setAttrib(imgElm, "title", data.title);
					dom.setHTML(imgElm, data.title);
				} else {
					editor.insertContent('<p class="system-article" title="'+data.title+'">'+data.title+'</p>&nbsp;');
				}
	
			}
		});
	}
	
	editor.addButton('eqdkp_pageobject', {
		icon: 'pageobject',
		tooltip: 'Insert Pageobject',
		onclick: insertPageobject,
		stateSelector: 'p.system-article'
	});
	
	
	editor.addMenuItem('eqdkp_pageobject', {
		icon: 'pageobject',
		text: 'Insert Pageobject',
		onclick: insertPageobject,
		context: 'insert',
		prependToContext: true
	});

});